@@ -17,6 +17,8 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <cmath>
+
 #include <Lum/Audio/PlaySound.h>
 
 #include <Lum/Base/L10N.h>
@@ -68,21 +70,227 @@ static Lum::Def::AppInfo info;
 typedef Lum::Model::StdTable<Status::Network> NetworkModel;
 typedef Lum::Base::Reference<NetworkModel>    NetworkModelRef;
 
+class SignalLevel : public Lum::Control
+{
+private:
+  Lum::Model::DoubleRef model;
+  Lum::OS::FontRef      font;
+  Lum::OS::ColorRef     red;
+  Lum::OS::ColorRef     green;
+
+public:
+  SignalLevel()
+  : font(Lum::OS::display->GetFont()),
+    red(1,0,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+    green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
+  {
+    // no code
+  }
+
+  bool SetModel(Lum::Base::Model* model)
+  {
+    this->model=dynamic_cast<Lum::Model::Double*>(model);
+
+    Control::SetModel(this->model);
+
+    return this->model.Valid();
+  }
+
+  void CalcSize()
+  {
+    minWidth=2*Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder)+
+             3*font->StringWidth(L"m",Lum::OS::Font::normal);
+    minHeight=font->pixelHeight+2*Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
+
+    width=minWidth;
+    height=minHeight;
+
+    Control::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Control::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    if (!model.Valid() || model->IsNull()) {
+      return;
+    }
+
+    double percent=model->Get();
+
+    if (percent<=0.2) {
+      draw->PushForeground(red);
+    }
+    else {
+      draw->PushForeground(green);
+    }
+
+    draw->FillRectangle(this->x,this->y+1,
+                        (size_t)(width*percent),height-2);
+
+
+    draw->PopForeground();
+
+    Lum::OS::FontRef    font=Lum::OS::display->GetFont();
+    std::wstring        label=Lum::Base::NumberToWString(lround(percent*100))+L"%";
+    Lum::OS::FontExtent extent;
+
+    font->StringExtent(label,extent);
+
+    draw->PushForeground(Lum::OS::Display::textColor);
+    draw->PushFont(font,0);
+    draw->DrawString(this->x+(width-extent.width+extent.left+extent.right)/2,
+                     this->y+(height-font->height)/2+font->ascent,
+                     label);
+    draw->PopFont();
+    draw->PopForeground();
+  }
+
+  void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
+  {
+    if (model==this->model) {
+      Redraw();
+    }
+    Control::Resync(model,msg);
+  }
+
+  static SignalLevel* Create(bool horizFlex, bool vertFlex)
+  {
+    SignalLevel *control=new SignalLevel();
+
+    control->SetFlex(horizFlex,vertFlex);
+
+    return control;
+  }
+};
+
+class Encryption : public Lum::Control
+{
+private:
+  Lum::Model::SizeTRef model;
+  Lum::OS::FontRef     font;
+  Lum::OS::ColorRef    grey;
+  Lum::OS::ColorRef    green;
+  Lum::OS::ColorRef    yellow;
+  Lum::OS::ColorRef    red;
+
+public:
+  Encryption()
+  : font(Lum::OS::display->GetFont()),
+    grey(0.7,0.7,0.7,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+    green(0,1,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+    yellow(1,0.5,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor)),
+    red(1,0,0,Lum::OS::display->GetColor(Lum::OS::Display::tableTextColor))
+  {
+    // no code
+  }
+
+  bool SetModel(Lum::Base::Model* model)
+  {
+    this->model=dynamic_cast<Lum::Model::SizeT*>(model);
+
+    Control::SetModel(this->model);
+
+    return this->model.Valid();
+  }
+
+  void CalcSize()
+  {
+    minHeight=font->pixelHeight+2*Lum::OS::display->GetSpaceVertical(Lum::OS::Display::spaceObjectBorder);
+    minWidth=minHeight;
+
+    width=minWidth;
+    height=minHeight;
+
+    Control::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Control::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    if (!model.Valid() || model->IsNull()) {
+      draw->PushForeground(grey);
+      draw->FillRectangle(this->x+(width-height)/2,this->y+1,
+                          height,height-2);
+      draw->PopForeground();
+      return;
+    }
+
+    size_t state=model->Get();
+
+    switch (state) {
+    case 0:
+      draw->PushForeground(green);
+      break;
+    case 1:
+      draw->PushForeground(yellow);
+      break;
+    case 2:
+      draw->PushForeground(red);
+      break;
+    default:
+      assert(false);
+    }
+
+    draw->FillRectangle(this->x+(width-height)/2,this->y+1,
+                        height,height-2);
+    draw->PopForeground();
+  }
+
+  void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
+  {
+    if (model==this->model) {
+      Redraw();
+    }
+    Control::Resync(model,msg);
+  }
+
+  static Encryption* Create(bool horizFlex, bool vertFlex)
+  {
+    Encryption *control=new Encryption();
+
+    control->SetFlex(horizFlex,vertFlex);
+
+    return control;
+  }
+};
+
 class NetworkDataProvider : public NetworkModel::DataProvider
 {
 private:
-  Lum::FuelGauge        *gauge;
-  Lum::Model::DoubleRef data;
+  SignalLevel           *gauge;
+  Encryption            *encrypt;
+  Lum::Model::DoubleRef signalLevel;
+  Lum::Model::SizeTRef  encryption;
 
 public:
   NetworkDataProvider()
-  : gauge(new Lum::FuelGauge()),
-    data(new Lum::Model::Double())
+  : gauge(SignalLevel::Create(true,false)),
+    encrypt(Encryption::Create(true,false)),
+    signalLevel(new Lum::Model::Double()),
+    encryption(new Lum::Model::SizeT())
   {
-    gauge->SetFlex(true,false);
+    signalLevel->SetRange(0,1);
+    gauge->SetModel(signalLevel);
 
-    data->SetRange(0,100);
-    gauge->SetModel(data);
+    encrypt->SetModel(encryption);
   }
 
   virtual ~NetworkDataProvider()
@@ -102,31 +310,6 @@ public:
         else {
           return L"?";
         }
-    case 3:
-        if (iter->encryption!=0) {
-          std::wstring encryption;
-
-          if (iter->encryption & Status::cryptNone) {
-            encryption.append(L"N");
-          }
-          if (iter->encryption & Status::cryptWEP) {
-            encryption.append(L"W");
-          }
-          if (iter->encryption & Status::cryptWPA_PSK) {
-            encryption.append(L"P");
-          }
-          if (iter->encryption & Status::cryptWPA_EAP) {
-            encryption.append(L"E");
-          }
-          if (iter->encryption & Status::cryptWPA2) {
-            encryption.append(L"2");
-          }
-
-          return encryption;
-        }
-        else {
-          return L"?";
-        }
     default:
       return L"";
     }
@@ -135,8 +318,30 @@ public:
   Lum::Object* GetObject(const NetworkModel::Iterator& iter, size_t column) const
   {
     switch (column) {
+    case 3:
+      if (iter->encryption!=0) {
+        if (iter->encryption & Status::cryptNone) {
+          encryption->Set(0);
+        }
+        else if (iter->encryption & Status::cryptWEP) {
+          encryption->Set(1);
+        }
+        else if (iter->encryption & Status::cryptWPA_PSK) {
+          encryption->Set(2);
+        }
+        else if (iter->encryption & Status::cryptWPA_EAP) {
+          encryption->Set(2);
+        }
+        else if (iter->encryption & Status::cryptWPA2) {
+          encryption->Set(2);
+        }
+      }
+      else {
+        encryption->SetNull();
+      }
+      return encrypt;
     case 4:
-      data->Set(iter->quality);
+      signalLevel->Set(iter->quality/100.0);
       return gauge;
     default:
       return NULL;
@@ -571,14 +776,20 @@ public:
       Lum::Table                         *table;
       Lum::Model::HeaderRef              headerModel;
       Lum::Model::SingleLineSelectionRef selection;
+      Lum::Def::Boolean                  scanNetworksDef(Lum::Def::Desc(L"Scan for networks"),
+                                                         scanForNetworks);
+      Lum::Def::Boolean                  wardriveDef(Lum::Def::Desc(L"Signal open networks"),
+                                                     wardrive);
 
       panel=Lum::VPanel::Create(true,true);
       panel->Add(Lum::HPanel::Create(true,false)
                  ->Add(Lum::Label::Create(true,false)
-                        ->AddLabel(L"Scan for networks:",Lum::Boolean::Create(scanForNetworks)))
+                        ->AddLabel(L"Scan for networks:",
+                                   Lum::OS::display->GetBehaviour()->GetBooleanControl(scanNetworksDef,true,false)))
                         ->AddSpace()
                         ->Add(Lum::Label::Create(true,false)
-                              ->AddLabel(L"Wardrive:",Lum::Boolean::Create(wardrive))));
+                              ->AddLabel(L"Wardrive:",
+                                         Lum::OS::display->GetBehaviour()->GetBooleanControl(wardriveDef,true,false))));
       panel->AddSpace();
 
       selection=new Lum::Model::SingleLineSelection;
